<?php
/*****************************************
* File      :   $RCSfile: class.newsletter.recipients.php,v $
* Project   :   Contenido
* Descr     :   Newsletter recipient class
* Modified  :   $Date: 2005/09/29 19:56:42 $
*
*  four for business AG, www.4fb.de, generated by HerrB (01.08.2004)
*
* $Id: class.newsletter.recipients.php,v 1.2 2005/09/29 19:56:42 timo.hummel Exp $
******************************************/

cInclude("classes", "class.newsletter.groups.php");

/**
 * Recipient management class
 */
class RecipientCollection extends ItemCollection {
	
	/**
	* Constructor Function
	* @param none
	*/
	function RecipientCollection() {
		global $cfg;
		parent::ItemCollection($cfg["tab"]["news_rcp"], "idnewsrcp");
		$this->_setItemClass("Recipient");
	}

	/**
	* Loads an item by its ID (primary key)
	* @param $itemID integer Specifies the item ID to load
	*/	
	function loadItem ($itemID) {
		$item = new Recipient();
		$item->loadByPrimaryKey($itemID);
		return ($item);
	}

	/**
	* Creates a new recipient
	* @param $email string specifies the e-mail adress
	* @param $name string specifies the recipient name (optional)
	* @param $confirmed integer specifies, if the recipient is confirmed (optional)
        * @param $joinID string specifies additional recipient group ids to join (optional, e.g. 47,12,...)
	*/		
	function create ($email, $name = "", $confirmed = 0, $joinID = "") {
		global $client, $lang, $auth;
		
		/* Check if the e-mail adress already exists */
		$email = strtolower($email); // e-mail always lower case
		$this->select("idclient='$client' AND idlang='$lang' AND email='".urlencode($email)."'");

		if ($this->next()) {
			return $this->create($email."_".substr(md5(rand()),0,10), $name, 0); /* 0: Deactivate 'confirmed' */
		}
		$item = parent::create();
		$item->set("idclient", $client);
		$item->set("idlang", $lang);
		$item->set("name", $name);
		$item->set("email", $email);
		$item->set("hash", substr(md5(rand()),0,17) . uniqid("")); /* Generating UID, 30 characters */
		$item->set("confirmed", $confirmed);
		
		if ($confirmed) {
			$item->set("confirmeddate", date("Y-m-d H:i:s"), false);
		}
		$item->set("deactivated", 0);
		$item->set("created", date("Y-m-d H:i:s"), false);
  		$item->set("author", $auth->auth["uid"]);
  		
		$item->store();

		$idrecipient = $item->get("idnewsrcp"); /* Getting internal id of new recipient */
		
		/* Put this recipient into the default recipient group (if any)*/
		$rgroups = new RecipientGroupCollection;
		$members = new RecipientGroupMemberCollection;

		$rgroups->select("idclient = '$client' AND idlang = '$lang' and defaultgroup='1'");
			
		while ($rgroup = $rgroups->next()) {
			$idgroup = $rgroup->get("idnewsgroup");
			$members->create($idgroup, $idrecipient);
		}

		/* Add to other recipient groups as well? Do so! */
		if ($joinID != "") {
			$arrJoinID = explode(",", $joinID);

			if (count($arrJoinID) > 0) {
				foreach ($arrJoinID as $idgroup) {
					$members->create($idgroup, $idrecipient);
				}		
			}
		}
		
		return $item;
	}

	/*
	* Overridden delete method to remove recipient from groupmember table
	* before deleting recipient
	*
	* @param $itemID int specifies the recipient
	*/		
	function delete ($itemID) {
		$associations = New RecipientGroupMemberCollection;
		$associations->select("idnewsrcp = '$itemID'");
		
		While ($item = $associations->next()) {
			$associations->delete($item->get("idnewsgroupmember"));	
		}
		parent::delete($itemID);	
	}

	/*
	* Purge method to delete recipients which hasn't been confirmed since over a month
	* @param  $timeframe int	Days after creation a not confirmed recipient will be removed
	* @return int 			Count of deleted recipients	
	*/		
	function purge ($timeframe) {
		global $client, $lang;
		
		$oRecipientCollection = new RecipientCollection;

		// DATEDIFF(created, NOW()) > 30 would be better, but it's only available in MySQL V4.1.1 and above
		// Note, that, as even this is 'generic db' TO_DAYS or NOW may not be available
		// in other database systems than MySQL
		//$this->select("idclient='$client' AND idlang='$lang' AND confirmed='0' AND (TO_DAYS(NOW()) - TO_DAYS(created)) > ".$timeframe);
		$oRecipientCollection->select("idclient='$client' AND idlang='$lang' AND confirmed='0' AND (TO_DAYS(NOW()) - TO_DAYS(created)) > ".$timeframe);
		
		while ($item = $oRecipientCollection->next()) {
			$oRecipientCollection->delete($item->get("idnewsrcp"));
		}
		return $oRecipientCollection->count();
	}

	/* Handling changed, old function for safety only, can be discarded after a while...
	function purge ($timeframe)
	{
		global $client, $lang;
		
		// DATEDIFF(created, NOW()) > 30 would be better, but it's only available in MySQL V4.1.1 and above
		// Note, that, as even this is 'generic db' TO_DAYS or NOW may not be available
		// in other database systems than MySQL
		$this->select("idclient='$client' AND idlang='$lang' AND confirmed='0' AND (TO_DAYS(NOW()) - TO_DAYS(created)) > ".$timeframe);
		
		while ($item = $this->next()) {
			$this->delete($item->get("idnewsrcp"));
		}
		return $this->count();
	} */

	/*
	* checkEMail returns true, if there is no recipient with the same e-mail address; otherwise false
	* @param  $email string	e-mail
	* @return recpient item if item with e-mail exists, false otherwise
	*/		
	function emailExists ($sEmail) {
		global $client, $lang;
		
		$oRecipientCollection = new RecipientCollection;

		$oRecipientCollection->setWhere("idclient", $client);
		$oRecipientCollection->setWhere("idlang", $lang);  
      		$oRecipientCollection->setWhere("email", strtolower($sEmail));
      		$oRecipientCollection->query();
      
      		if ($oItem = $oRecipientCollection->next()) {
			return $oItem;
		} else {
			return false;
		}
	}

	/**
	* Sets a key for all recipients without key or an old key (len(key) <> 30)
	* No parameters
	*/		
	function updateKeys() {
		// global $client, $lang;
		
		// $this->select("idclient='$client' AND idlang='$lang' AND LENGTH(hash) <> 30");
		$this->select("LENGTH(hash) <> 30");
		
		$updated = $this->count();
		while ($item = $this->next()) {
			$item->set("hash", substr(md5(rand()),0,17) . uniqid("")); /* Generating UID, 30 characters */  		
			$item->store();
		}
		
		return $updated;
	}
}

/**
 * Single Recipient Item
 */
class Recipient extends Item {
	
	/**
	* Constructor Function
	* @param string $table The table to use as information source
	*/
	function Recipient() {
		global $cfg;
		
		parent::Item($cfg["tab"]["news_rcp"], "idnewsrcp");
	}
	
	/**
	* Checks if the given md5 matches the md5(email) in the database
	* @param $md5email string md5 of E-Mail to check
	* @return boolean True if the hash matches, false otherwise
	*/		
	function checkMD5Email ($md5email) {
		if ($md5email == md5($this->get("email"))) {
			return true;
		} else {
			return false;
		}
	}
	
	function store () {
		global $auth;
		
		$this->set("lastmodified", date("Y-m-d H:i:s"), false);
  		$this->set("modifiedby", $auth->auth["uid"]);
  		parent::store();
	}	
}

?>