<?php
/*****************************************
* File      :   $RCSfile: class.module.php,v $
* Project   :   Contenido
* Descr     :   Module access classes
*
* Author    :   Timo A. Hummel
*               
* Created   :   26.02.2003
* Modified  :   $Date: 2006/10/05 23:43:48 $
*
*  four for business AG, www.4fb.de
*
* $Id: class.module.php,v 1.13 2006/10/05 23:43:48 bjoern.behrens Exp $
******************************************/

cInclude("classes", "class.genericdb.php");

class cApiModuleCollection extends ItemCollection {
	
	/**
     * Constructor Function
     * @param none
     */
	function cApiModuleCollection()
	{
		global $cfg;
		parent::ItemCollection($cfg["tab"]["mod"], "idmod");
		
		$this->_setItemClass("cApiModule");
	}

	/**
     * Creates a new communication item
     */		
	function create ($name)
	{
		global $auth, $client;
		$item = parent::create();
		
		$item->set("idclient", $client);
		$item->set("name", $name);
		$item->set("author", $auth->auth["uid"]);
		$item->set("created", date("Y-m-d H:i:s"),false);
		$item->store();
		return $item;
	}
}

/**
 * Module access class
 */
class cApiModule extends Item {

	var $_error;
	
    /**
     * Constructor Function
     * @param $loaditem Item to load
     */
    function cApiModule($loaditem = false) {

		global $cfg;
		parent::Item($cfg["tab"]["mod"], "idmod");
		
		// Using no filters is just for compatibility reasons.
		// That's why you don't have to stripslashes values if you store them
		// using ->set. You have to add slashes, if you store data directly
		// (data not from a form field)
		$this->setFilters(array(), array());
		
		if ($loaditem !== false)
		{
			$this->loadByPrimaryKey($loaditem);	
		}
    } // end function

	function loadByPrimaryKey ($id)
	{
		parent::loadByPrimaryKey($id);
		
		if ($this->_shouldLoadFromFiles())
		{
			global $cfg;
			$sRootPath = $cfg['path']['contenido'] . $cfg['path']['modules'] . $this->get("idclient")."/" . $this->get("idmod").".xml";

			if (file_exists($sRootPath))
			{
				$this->import($sRootPath);		
			}
			
		}
	}

	/**
     * getTranslatedName
	 * Returns the translated name of the module if
	 * a translation exists.
	 *
     * @param none
	 * @return string Translated module name or original
     */		
	function getTranslatedName ()
	{
		global $lang;
		
		/* If we're not loaded, return */
    	if ($this->virgin == true)
    	{
    		return false;
    	}		
		
		$modname = $this->getProperty("translated-name", $lang);
		
		if ($modname === false)
		{
			return $this->get("name");	
		} else {
			return $modname;
		}
	}
	
	/**
     * setTranslatedName
	 * Sets the translated name of the module
	 *
     * @param $name string Translated name of the module
	 * @return none
     */		
	function setTranslatedName ($name)
	{
		global $lang;
		
		$this->setProperty("translated-name", $lang, $name);	
	}

	/**
     * parseModuleForStrings
	 * Parses the module for mi18n strings and returns
	 * them in an array
	 *
	 * @return array Found strings for this module
     */		
    function parseModuleForStrings ()
    {
    	if ($this->virgin == true)
    	{
    		return false;
    	}
    	
		/* Fetch the code, append input to output */
		$code  = $this->get("output");
		$code .= $this->get("input");
		
		/* Initialize array */
    	$strings = array();
    	
    	/* Split the code into mi18n chunks */
    	$varr = preg_split('/mi18n([\s]*)\(([\s]*)"/', $code, -1);
    	
		if (count($varr) > 1)
    	{
        	foreach ($varr as $key => $value)
        	{
	        	/* Search first closing */
	        	$closing = strpos($value,'")');
	        	
	        	if ($closing === false)
	        	{
	        		$closing = strpos($value,'" )');	
	        	}
	        	
	        	if ($closing !== false)
	        	{
	        		$value = substr($value,0, $closing).'")';
	        	}
	        	
	        	/* Append mi18n again */
	        	$varr[$key] = 'mi18n("'.$value;
	        
	        	/* Parse for the mi18n stuff */
	        	preg_match_all('/mi18n([\s]*)\("(.*)"\)/', $varr[$key], $results);
	    		
	    		/* Append to strings array if there are any results */
	    		if (is_array($results[1]) && count($results[2]) > 0)
	    		{
	    			$strings = array_merge($strings, $results[2]);	
	    		}
	    		
	    		/* Unset the results for the next run */
	        	unset($results);
	        }
    	}
        
        /* Make the strings unique */
        return array_unique($strings);
    }

    /**
     * moduleInUse()
     * Checks if the module is in use
     * @return bool    Specifies if the module is in use
     */
    function moduleInUse( $module ) {
        global $cfg;


        $db = new DB_Contenido;

        $sql = "SELECT
                    idmod
                FROM
                ". $cfg["tab"]["container"] ."
                WHERE
                    idmod = '".$module."'";

        $db->query($sql);

        if ($db->nf() == 0)
        {
            return false;
        } else {
            return true;
        }


    } // end function  

    /**
     * isOldModule()
     * Checks if the module is a pre-4.3 module
     * @return boolean true if this module is an old one
     */
	function isOldModule ()
	{
		/* Keywords to scan */
		$scanKeywords = array('$cfgTab', 'idside', 'idsidelang');
		
		$input  = $this->get("input");
		$output = $this->get("output");
		
		foreach ($scanKeywords as $keyword)
		{
			if (strstr($input, $keyword))
			{
				return true;	
			}
			
			if (strstr($output, $keyword))
			{
				return true;
			}
		}
	}
	
	function getField ($field)
	{
		$value = parent::getField($field);
		
		switch ($field)
		{
			case "name":
				if ($value == "")
				{
					$value = i18n("- Unnamed Module -");	
				}	
		}
		
		return ($value);
	}
	
	function import ($file)
    {
    	global $_mImport;
    	
    	cInclude("classes", "class.xmlparser.php");
    	
    	$parser = new XmlParser;
    	
    	
    	$parser->setEventHandlers(array("/module/name"=> "cModule_in_Handler",
    									"/module/description"=> "cModule_in_Handler",
    									"/module/type"=> "cModule_in_Handler",
    									"/module/input" => "cModule_in_Handler",
    									"/module/output" => "cModule_in_Handler"));  
    	
    	if ($parser->parseFile($file))
    	{
    		$bStore = false;
			foreach ($_mImport as $key => $value)
			{
				if ($this->get($key) != $value)
				{
					$this->set($key, addslashes($value));
					$bStore = true;
				}
			}
			
			if ($bStore == true)
			{
				$this->store();
			}
			return true;
    	} else {
    		$this->_error = $parser->error;
    		return false;	
    	}
		
		
   		
    }
    
    function store ()
    {
    	global $cfg;
    	
		if (getEffectiveSetting("modules", "disable-history", "false") !== "true")
		{
			$modarchives = new cApiModuleHistoryCollection;
			$modarchives->create($this->get("idmod"));	
		}
    	parent::store();
    	
		conGenerateCodeForAllArtsUsingMod($this->get("idmod"));
		
		
    	
    	if ($this->_shouldStoreToFile())
    	{
    		if ($this->_makeFileDirectoryStructure())
    		{
	    		$sRootPath = $cfg['path']['contenido'] . $cfg['path']['modules'] . $this->get("idclient")."/";
	    		file_put_contents($sRootPath . $this->get("idmod").".xml", $this->export($this->get("idmod").".xml", true));
    		}
    	}
    }
    
    function _makeFileDirectoryStructure ()
    {
    	global $cfg;
    	
    	$sRootPath = $cfg['path']['contenido'] . $cfg['path']['modules']; 
    	if (!is_dir($sRootPath))
    	{
    		@mkdir($sRootPath);
    	}
    	
    	$sRootPath = $cfg['path']['contenido'] . $cfg['path']['modules'] . $this->get("idclient")."/";
    	if (!is_dir($sRootPath))
    	{
    		@mkdir($sRootPath);	
    	}
    	
    	if (is_dir($sRootPath))
    	{
    		return true;	
    	} else {
    		return false;	
    	}
    	
    }
    
    function _shouldStoreToFile ()
    {
    	if (getSystemProperty("modules", "storeasfiles") == "true")
    	{
    		return true;	
    	} else {
    		return false;	
    	}
    }
    
    function _shouldLoadFromFiles ()
    {
    	if (getSystemProperty("modules", "loadfromfiles") == "true")
    	{
    		return true;	
    	} else {
    		return false;	
    	}
    }
    
	/**
	 * export
     * Exports the specified module strings to a file
	 *
	 * @param $idmod    int Module ID
	 * @param $idlang   int Language ID
	 * @param $filename string Filename to return
	 * @param $return	boolean if false, the result is immediately sent to the browser 
     */    
    function export ($filename, $return = false)
    {
    	cInclude("classes", "class.xmltree.php");
    	$tree  = new XmlTree('1.0', 'ISO-8859-1');
    	$root =& $tree->addRoot('module');

		$root->appendChild("name", htmlspecialchars($this->get("name")));    		
		$root->appendChild("description", htmlspecialchars($this->get("description")));
		$root->appendChild("type", htmlspecialchars($this->get("type")));
		$root->appendChild("input", htmlspecialchars($this->get("input")));
		$root->appendChild("output", htmlspecialchars($this->get("output")));
		
    	if ($return == false)
    	{
			header("Content-Type: text/xml");
    		header("Etag: ".md5(mt_rand()));
    		header("Content-Disposition: attachment;filename=\"$filename\"");
    		$tree->dump(false);
    	} else {
    		return stripslashes($tree->dump(true));	
    	}	
    }		

} // end class



class cApiModuleTranslationCollection extends ItemCollection
{

	var $_error;
		
	/**
     * Constructor Function
     * @param none
     */
	function cApiModuleTranslationCollection()
	{
		global $cfg;
		parent::ItemCollection($cfg["tab"]["mod_translations"], "idmodtranslation");
		
		$this->_setItemClass("cApiModuleTranslation");
	}

	/**
     * Creates a new module translation item
     */		
	function create ($idmod, $idlang, $original, $translation = false)
	{
		/* Check if the original already exists. If it does, 
		   update the translation if passed */
		$mod = new cApiModuleTranslation;
		$sorg = $mod->_inFilter($original);
		
		$this->select("idmod = '$idmod' AND idlang = '$idlang' AND original = '$sorg'");
		
		if ($item = $this->next())
		{
			if ($translation !== false)
			{
				$item->set("translation", $translation);
				$item->store();
			}
			return $item;	
		} else {
			
			$item = parent::create();
			$item->set("idmod", $idmod);
			$item->set("idlang", $idlang);
			$item->set("original", $original);
			$item->set("translation", $translation);
			$item->store();		
			return $item;
		}
	}
	
	/**
	 * fetchTranslation
     * Fetches a translation
	 *
	 * @param $module int Module ID
	 * @param $lang   int Language ID
	 * @param $string string String to lookup
     */
    function fetchTranslation ($module, $lang, $string)
    {
		/* If the f_obj does not exist, create one */
    	if (!is_object($this->f_obj))
    	{
    		$this->f_obj = new cApiModuleTranslation;
    	}
    	
    	/* Create original string */
    	$sorg = $this->f_obj->_inFilter($string);
    	
    	/* Look up */
		$this->select("idmod = '$module' AND idlang='$lang' AND original = '$sorg'");
		
		if ($t = $this->next())
		{
			$translation = $t->get("translation");
			
			if ($translation != "")
			{
				return $translation;
			} else {
				return $string;
			}
		} else {
			return $string;
		}
    }
    

    	    
    function import ($idmod, $idlang, $file)
    {
    	global $_mImport;
    	
    	cInclude("classes", "class.xmlparser.php");
    	
    	$parser = new XmlParser;
    	
    	
    	

    	
    	$parser->setEventHandlers(array("/module/translation/string/original"=> "cModule_in_moduleHandlerOriginal",
    									"/module/translation/string/translation"=> "cModule_in_moduleHandlerTranslated"));  
    	
    	if ($parser->parseFile($file))
    	{
			foreach ($_mImport["items"] as $key => $value)
			{
				$this->create ($idmod, $idlang, $key, $value);
			}
			
			return true;
			
    	} else {
    		$this->_error = $parser->error;
    		return false;	
    	}
   		
    }
	/**
	 * export
     * Exports the specified module strings to a file
	 *
	 * @param $idmod    int Module ID
	 * @param $idlang   int Language ID
	 * @param $filename string Filename to return
	 * @param $return	boolean if false, the result is immediately sent to the browser 
     */    
    function export ($idmod, $idlang, $filename, $return = false)
    {
    	$langobj = new Language;
        $langobj->loadByPrimaryKey($idlang);
        
        $langstring = $langobj->get("name") . ' ('.$idlang.')';
    	
    	$translations = new cApiModuleTranslationCollection;
    	$translations->select("idmod = '$idmod' AND idlang='$idlang'");
    	
    	cInclude("classes", "class.xmltree.php");
    	
    	$tree  = new XmlTree('1.0', 'ISO-8859-1');
    	$root =& $tree->addRoot('module');
    		
        $translation =& $root->appendChild('translation');
       	$translation->setNodeAttribs(array("origin-language-id" => $idlang,
       									   "origin-language-name" => $langobj->get("name")));
    	   									   
    	while ($otranslation = $translations->next())
    	{
    		$string =&$translation->appendChild("string");
    		
    		$string->appendChild("original", htmlspecialchars($otranslation->get("original")));
    		$string->appendChild("translation", htmlspecialchars($otranslation->get("translation")));
    		
    	}	   									   
    	
    	if ($return == false)
    	{
			header("Content-Type: text/xml");
    		header("Etag: ".md5(mt_rand()));
    		header("Content-Disposition: attachment;filename=\"$filename\"");
    		$tree->dump(false);
    	} else {
    		return $tree->dump(true);	
    	}	
    }	
	
}

/**
 * Module access class
 */
class cApiModuleTranslation extends Item {

    /**
     * Constructor Function
     * @param $loaditem Item to load
     */
    function cApiModuleTranslation($loaditem = false)
    {
		global $cfg;
		parent::Item($cfg["tab"]["mod_translations"], "idmodtranslation");
		
		if ($loaditem !== false)
		{
			$this->loadByPrimaryKey($loaditem);	
		}
    } // end function
   

} // end class

function cModule_in_Handler($name, $attribs, $content)
{
	global $_mImport;
	
	$_mImport[$name] = $content;
}
    	
function cModule_in_moduleHandlerOriginal($name, $attribs, $content)
{
	global $_mImport;
	
	$_mImport["currentItem"] = $content;
	
}

function cModule_in_moduleHandlerTranslated($name, $attribs, $content)
{
	global $_mImport;
	
	$_mImport["items"][$_mImport["currentItem"]] = $content;
	
}
?>
