<?php
cInclude("classes",  "class.wysiwyg_editor.php");
cInclude("includes", "functions.lang.php");

class cTinyMCEEditor extends cWYSIWYGEditor
{
	var $_sLanguage;
	var $_sBaseURL;
	var $_sDocumentBaseURL;
	var $_sStyles;
	var $_sStylesheet;
	var $_iWidth;
	var $_iHeight;
	var $_sToolbarMode;
	var $_sLists;
	var $_sTextDirection; 
	var $_sToolbarAlign;
	var $_bXHTMLMode = false;
	
	function cTinyMCEEditor ($sEditorName, $sEditorContent)
	{
		global $belang, $cfg, $cfgClient,$client, $lang;
		
		cWYSIWYGEditor::cWYSIWYGEditor($sEditorName, $sEditorContent);
		$this->_setEditor("tinymce2");	
		
		$langs = i18nGetAvailableLanguages();

		$this->setEditorLanguage($langs[$belang][4]);
		
		$this->setBaseURL(preg_replace('/^https?:\/\/[^\/]+(.*)$/', '$1', $this->getEditorPath()));
		$this->setDocumentBaseURL($cfgClient[$client]["path"]["htmlpath"]);
		
		$editor_stylesheet = getEffectiveSetting("wysiwyg","tinymce-stylesheet-file", "");

		if ($editor_stylesheet == "" || $editor_stylesheet == "false")
		{
			$editor_stylesheet = "";
		} else {
		   /* quick and dirty hack */
		   if (!preg_match('/^(http|https):\/\/((?:[a-zA-Z0-9_-]+\.?)+):?(\d*)/', $editor_stylesheet)) {
			   if (preg_match('/^\//', $editor_stylesheet)) {
				   $editor_stylesheet = "http://".$_SERVER['HTTP_HOST'].$editor_stylesheet;
			   } else {
				   $editor_stylesheet = $cfgClient[$client]["htmlpath"]["frontend"].$editor_stylesheet;
			   }
		   }
		}
		
		$this->setStylesheet($editor_stylesheet);
		
		$lists = getEffectiveSetting("wysiwyg","tinymce-lists", "");
		if ($lists !== "") {
			$this->setLists($lists);
		}
		
		// enable user defined styles
		$editor_styles = getEffectiveSetting("wysiwyg","tinymce-styles", "");
		if ($editor_styles !== "") {
			// just to be sure that spaw-styles syntax also works
			$editor_styles = preg_replace('/;$/i', '', str_replace("|", "=", trim($editor_styles)));
			$this->setStyles($editor_styles);
		}
		
		$editor_width  = getEffectiveSetting("wysiwyg","tinymce-width", "100%");
		$editor_height = getEffectiveSetting("wysiwyg","tinymce-height", "480");

		$this->setWidth($editor_width);
		$this->setHeight($editor_height);
		
		$editor_toolbar_mode = trim(strtolower(getEffectiveSetting("wysiwyg","tinymce-toolbar-mode", "full")));
		$this->setToolbarMode ($editor_toolbar_mode);
		
		$editor_xhtml_style = getEffectiveSetting("generator", "xhtml", false);
		
		$toolbar1 = getEffectiveSetting("wysiwyg", "tinymce-toolbar1", false);
		$toolbar2 = getEffectiveSetting("wysiwyg", "tinymce-toolbar2", false);
		$toolbar3 = getEffectiveSetting("wysiwyg", "tinymce-toolbar3", false);
		$plugins  = getEffectiveSetting("wysiwyg", "tinymce-plugins", false);
		
		if ($editor_toolbar_mode == "custom")
		{
			$this->setCustomToolbar($toolbar1, $toolbar2, $toolbar3, $plugins);	
		}
		
		$this->setXHTMLMode($editor_xhtml_style);
		
		$this->setDateformat(getEffectiveSetting("backend", "timeformat_date", "Y-m-d"));
		$this->setTimeformat(getEffectiveSetting("backend", "timeformat_time", "H:i:s"));
		
		$this->setTextDirection(langGetTextDirection($lang)); // sets also ToolbarAlign
	}
	
	function setToolbarMode ($sMode)
	{
		$this->_sToolbarMode = $sMode;
	}
	
	function setXHTMLMode ($bEnabled = true)
	{
		$this->_bXHTMLMode = $bEnabled;	
	}
	
	function setCustomToolbar ($toolbar1, $toolbar2, $toolbar3, $plugins)
	{
		$this->_sCustomToolbar = array();
		$this->_sCustomToolbar["toolbar1"] = $toolbar1;
		$this->_sCustomToolbar["toolbar2"] = $toolbar2;
		$this->_sCustomToolbar["toolbar3"] = $toolbar3;
		$this->_sCustomToolbar["plugins"] = $plugins;
		
		$this->setToolbarMode("custom");
	}
	
	function setStylesheet ($sStylesheet)
	{
		$this->_sStylesheet = $sStylesheet;
	}
	
	function setStyles ($sStyles)
	{
		$this->_sStyles = $sStyles;
	}

	function setLists($sLists)
	{
		$this->_sLists = strtolower(str_replace(" ", "", $sLists));
	}

	function setWidth ($iWidth)
	{
		$this->_iWidth = $iWidth;	
	}
		
	function setHeight ($iHeight)
	{
		$this->_iHeight = $iHeight;	
	}
	
	function setDocumentBaseURL ($sDocumentBaseURL)
	{
		$this->_sDocumentBaseURL = $sDocumentBaseURL;	
	}
	
	function setBaseURL ($sBaseUrl)
	{
		$this->_sBaseURL = $sBaseUrl;	
	}

	function setEditorLanguage ($sLanguage)
	{
		if ($sLanguage != "")
		{
			$this->_sLanguage = $sLanguage;	
		}	
	}
	
	function setTextDirection ($sTextDirection)
	{
		$this->_sTextDirection = $sTextDirection;
		$this->setToolbarAlign (($sTextDirection == 'rtl') ? 'right' : 'left');
	}

	function setToolbarAlign ($sToolbarAlign)
	{
		if ($sToolbarAlign == 'right')
		{
			$this->_sToolbarAlign = 'right';
		} else
		{
			$this->_sToolbarAlign = 'left';
		}
	}
		
	function getScripts ()
	{
		$return = '<script language="javascript" type="text/javascript" src="'.$this->_sBaseURL.'jscripts/tiny_mce/tiny_mce.js"></script>';
		
		return ($return);	
	}
	
	function setTimeformat ($sTimeformat)
	{	
		$this->_sTimeformat = $this->convertFormat($sTimeformat);
	}
	
	function convertFormat ($sInput)
	{
		$aFormatCodes = array(	"y" => "%y",
								"Y" => "%Y",
								"d" => "%d",
								"m" => "%m",
								"H" => "%H",
								"h" => "%I",
								"i" => "%M",
								"s" => "%S",
								"a" => "%P",
								"A" => "%P");
		foreach ($aFormatCodes as $sFormatCode => $sReplacement)
		{
			$sInput = str_replace($sFormatCode, $sReplacement, $sInput);
		}
		
		return ($sInput);
		
	}
	
	function setDateformat ($sDateformat)
	{
		$this->_sDateformat = $this->convertFormat($sDateformat);
	}
	
	function getEditor ()
	{
		global $sess, $cfg, $lang, $client;
		
		switch ($this->_sToolbarMode)
		{
			case "full": /* show all options */
				$editor_buttons_1 = "cut,copy,paste,pastetext,pasteword,separator,search,replace,separator,undo,redo,separator,bold,italic,underline,strikethrough,sub,sup,separator,insertdate,inserttime,preview,separator,styleselect";
				$editor_buttons_2 = "link,unlink,anchor,image,flash,advhr,separator,bullist,numlist,separator,outdent,indent,separator,justifyleft,justifycenter,justifyright,justifyfull,removeformat,separator,forecolor,backcolor,separator,ltr,rtl,separator,fullscreen,visualaid,charmap,cleanup,separator,code";
				$editor_buttons_3 = "tablecontrols,separator,formatselect,fontselect,fontsizeselect";
				$editor_plugins   = "table,save,advhr,advimage,advlink,insertdatetime,preview,zoom,flash,searchreplace,print,contextmenu,paste,directionality,fullscreen";
				break;
		
			case "simple": /* does not show font and table options */
				$editor_buttons_1 = "cut,copy,paste,pastetext,pasteword,separator,search,replace,separator,undo,redo,separator,bold,italic,underline,strikethrough,sub,sup,separator,insertdate,inserttime,preview,separator,styleselect";
				$editor_buttons_2 = "link,unlink,anchor,image,flash,advhr,separator,bullist,numlist,separator,outdent,indent,separator,justifyleft,justifycenter,justifyright,justifyfull,removeformat,separator,forecolor,backcolor,separator,ltr,rtl,separator,visualaid,charmap,cleanup,separator,code";
				$editor_buttons_3 = "";
				$editor_plugins   = "advhr,advimage,advlink,insertdatetime,preview,zoom,flash,searchreplace,print,contextmenu,paste,directionality";
				break;
		
			case "mini": /* minimal options */
				$editor_buttons_1 = "undo,redo,separator,bold,italic,underline,strikethrough,separator,link";
				$editor_buttons_2 = "";
				$editor_buttons_3 = "";
				$editor_plugins   = "advlink";
				break;
			case "custom":
				$editor_buttons_1 = $this->_sCustomToolbar["toolbar1"]; 
				$editor_buttons_2 = $this->_sCustomToolbar["toolbar2"]; 
				$editor_buttons_3 = $this->_sCustomToolbar["toolbar3"];
				$editor_plugins   = $this->_sCustomToolbar["plugins"]; 
				break;
		   default: /* default options */
		      $editor_buttons_1 = "undo,redo,separator,bold,italic,underline,strikethrough,separator,link,unlink,anchor,image,flash,advhr,separator,tablecontrols";
		      $editor_buttons_2 = "styleselect,separator,bullist,numlist,separator,outdent,indent,separator,justifyleft,justifycenter,justifyright,justifyfull,removeformat,separator,forecolor,backcolor,separator,sub,sup,separator,code";
		      $editor_buttons_3 = "";
		      $editor_plugins = "table,advhr,advimage,advlink,flash,searchreplace,contextmenu,paste";
		}

		$browserparameters = array("restrict_imagebrowser" => array("jpg", "gif", "jpeg", "png"));
		$sess->register("browserparameters");	
			
		$tpl = new Template;
		$tpl->set('s', 'ADVANCED_STYLES', $this->_sStyles);
		$tpl->set('s', 'IMAGEBROWSER', $cfg["path"]["contenido_fullhtml"] .'frameset.php?area=upl&contenido='.$sess->id.'&appendparameters=imagebrowser');
		$tpl->set('s', 'FILEBROWSER',  $cfg["path"]["contenido_fullhtml"] .'frameset.php?area=upl&contenido='.$sess->id.'&appendparameters=filebrowser');
		$tpl->set('s', 'FLASHBROWSER', $cfg["path"]["contenido_fullhtml"] .'frameset.php?area=upl&contenido='.$sess->id.'&appendparameters=imagebrowser');
		$tpl->set('s', 'EDITORNAME', $this->_sEditorName);
		$tpl->set('s', 'EDITORLANG', $this->_sLanguage);
		$tpl->set('s', 'EDITORWIDTH', $this->_iWidth);
		$tpl->set('s', 'EDITORHEIGHT', $this->_iHeight);
		$tpl->set('s', 'EDITORPLUGINS', $editor_plugins);
		$tpl->set('s', 'BUTTONS1', $editor_buttons_1);
		$tpl->set('s', 'BUTTONS2', $editor_buttons_2);
		$tpl->set('s', 'BUTTONS3', $editor_buttons_3);
		$tpl->set('s', 'DOCUMENTBASEURL', $this->_sDocumentBaseURL);
		$tpl->set('s', 'CSSURL', $this->_sStylesheet);
		$tpl->set('s', 'INSERTDATE_DATEFORMAT', $this->_sDateformat);
		$tpl->set('s', 'INSERTDATE_TIMEFORMAT', $this->_sTimeformat);
		$tpl->set('s', 'DIRECTIONALITY', $this->_sTextDirection); 
		$tpl->set('s', 'TOOLBAR_ALIGN', $this->_sToolbarAlign);
		
		$aLists = explode(",", $this->_sLists);
		if (!$aLists) {
			$aLists = array();
		}
		
		if (in_array("link", $aLists)) { 
			$tpl->set('s', 'EXTERNAL_LINK_LIST', $this->_sBaseURL . "list.php?mode=link&lang=" . $lang . "&client=" . $client . "#");
		} else {
			$tpl->set('s', 'EXTERNAL_LINK_LIST', '');
		}
		
		if (in_array("image", $aLists)) {
			$tpl->set('s', 'EXTERNAL_IMAGE_LIST', $this->_sBaseURL . "list.php?mode=image&lang=" . $lang . "&client=" . $client . "#");
		} else {
			$tpl->set('s', 'EXTERNAL_IMAGE_LIST', '');
		}
		
		if (in_array("flash", $aLists)) {
			$tpl->set('s', 'EXTERNAL_FLASH_LIST', $this->_sBaseURL . "list.php?mode=flash&lang=" . $lang . "&client=" . $client . "#");
		} else {
			$tpl->set('s', 'EXTERNAL_FLASH_LIST', '');
		}
		
		if (!$this->_bXHTMLMode)
		{
			$tpl->set('s', 'CLEANUPCALLBACK', 'cleanup_callback : "CustomCleanupContent",');
		} else {
			$tpl->set('s', 'CLEANUPCALLBACK', '');			
		}
		
		$valid_elements = getEffectiveSetting('wysiwyg', 'tinymce-extended-valid-elements', 'form[name|action|method],textarea[name|style|cols|rows],input[type|name|value|style|onclick],a[name|href|target|title|onclick],img[class|src|border=0|alt|title|hspace|vspace|width|height|align|onmouseover|onmouseout|name],hr[class|width|size|noshade],font[face|size|color|style],span[class|align|style]');
		$tpl->set('s', 'EXTENDED_VALID_ELEMENTS', $valid_elements);

		$valid_elements = getEffectiveSetting('wysiwyg', 'tinymce-valid-elements', '+a[name|href|target|title],strong/b[class],em/i[class],strike[class],u[class],p[dir|class|align],ol,ul,li,br,img[class|src|border=0|alt|title|hspace|vspace|width|height|align],sub,sup,blockquote[dir|style],table[border=0|cellspacing|cellpadding|width|height|class|align],tr[class|rowspan|width|height|align|valign],td[dir|class|colspan|rowspan|width|height|align|valign],div[dir|class|align],span[class|align],pre[class|align],address[class|align],h1[dir|class|align],h2[dir|class|align],h3[dir|class|align],h4[dir|class|align],h5[dir|class|align],h6[dir|class|align],hr');
		$tpl->set('s', 'VALID_ELEMENTS', $valid_elements);
				
		$oTextarea = new cHTMLTextarea($this->_sEditorName, $this->_sEditorContent);
		$oTextarea->setId($this->_sEditorName);
		
		$bgColor = getEffectiveSetting("wysiwyg", "tinymce-backgroundcolor", "white");
		$oTextarea->setStyle("width:".$this->_iWidth."px; height:".$this->_iHeight."px; background-color: $bgColor;");

		$return = $tpl->generate($cfg['path']['all_wysiwyg'] . $this->_sEditor . "/tinymce.tpl.html", true);
		$return .= $oTextarea->render();
		
		return $return;
	}
}
?>