<?php

/******************************************
* File      :   functions.modrewrite.php
* Project   :   Contenido
* Descr     :   functions to upgrade to modrewrite
*
* Author    :   Stefan Seifarth
* Created   :   28.12.2005
* Modified  :   28.12.2005
*
*  www.polycoder.de
*/
error_reporting (E_ALL ^ E_NOTICE);
/**
 * upgradeModRewrite
 * 
 * handle upgrading of mod rewrite needed 
 * database table columns
 */
function upgradeModRewrite () {
   $db = new DB_Contenido($_SESSION["dbhost"], "", $_SESSION["dbuser"], $_SESSION["dbpass"]);

   if (checkMySQLDatabaseCreation($db, $_SESSION["dbname"])) {
	  $db = new DB_Contenido($_SESSION["dbhost"], $_SESSION["dbname"], $_SESSION["dbuser"], $_SESSION["dbpass"]);
   }
   
   // is setup a upgrade from an older
   // contenido version with installed
   // mod rewrite engine
   $older_version = isOldModRewrite();
   
   switch ($older_version) {
    
      case "4.4.x":
         $sql = "ALTER TABLE " . $_SESSION["dbprefix"] . "_art_lang CHANGE websafename urlname VARCHAR( 128 )";
         $db->query( $sql );
         break;
         
      case "4.5.x":
         break;
         
      default:
         $sql = "ALTER TABLE " . $_SESSION["dbprefix"] . "_art_lang ADD urlname VARCHAR( 128 ) AFTER title";
         $db->query( $sql );
         break;
   }
   
   // check for empty article fields
   $sql = "SELECT idlang, title, idart FROM " . $_SESSION["dbprefix"] . "_art_lang WHERE urlname IS NULL OR urlname = ''";
   $db->query($sql);
   while ( $db->next_record() ) {
      set_article($db->f('title'), $db->f('idart'), $db->f('idlang'));
   }
   
   // check for empty category fields
   $sql = "SELECT name, idcat, idlang FROM " . $_SESSION["dbprefix"] . "_cat_lang WHERE urlname IS NULL OR urlname = ''";
   $db->query($sql);
   while ( $db->next_record() ) {
      set_category($db->f('name'), $db->f('idcat'), $db->f('idlang'));
   }

}



/**
 * isOldModRewrite
 * 
 * check if an older ModRewrite Bundle is 
 * currently installed an return the version
 *
 * @return contenido mod rewrite bundle version
 */
function isOldModRewrite () {
   
   $db = new DB_Contenido($_SESSION["dbhost"], "", $_SESSION["dbuser"], $_SESSION["dbpass"]);

   if (checkMySQLDatabaseCreation($db, $_SESSION["dbname"])) {
	  $db = new DB_Contenido($_SESSION["dbhost"], $_SESSION["dbname"], $_SESSION["dbuser"], $_SESSION["dbpass"]);
   }
   
   $sql = "SHOW COLUMNS FROM %s LIKE %s";

   // check if contenido version 4.4.x
   $db->query( sprintf($sql, $_SESSION["dbprefix"] . "_art_lang", "websafename") );
   if ( $db->num_rows() > 0 ) {
      return "4.4.x";
   }
   
   // check if contenido version 4.5.x
   $db->query( sprintf($sql, $_SESSION["dbprefix"] . "_art_lang", "urlname") );
   if ( $db->num_rows() > 0 ) {
      return "4.5.x";
   }

   return false;
}


/**
 * set_article()
 *
 * set websafe name in article list
 *
 * insert new websafe name in article list 
 *
 * @param	string	original name (will be converted)
 * @param	integer	current article id
 * @param	integer current language id
 * @return 	boolean	true if insert was successfully
 */
function set_article ( $str_name = "", $int_id = 0, $int_lang_id = 0, $int_idcat = 0) {
   $db = new DB_Contenido($_SESSION["dbhost"], "", $_SESSION["dbuser"], $_SESSION["dbpass"]);

   if (checkMySQLDatabaseCreation($db, $_SESSION["dbname"])) {
	  $db = new DB_Contenido($_SESSION["dbhost"], $_SESSION["dbname"], $_SESSION["dbuser"], $_SESSION["dbpass"]);
   }
		
	// create websafe name
	$str_new_name = capiStrCleanURLCharacters($str_name);
		
	// check if websafe name already exists
	if ( in_articles($str_new_name, $int_id, $int_lang_id, $int_idcat) ) {
      // create new websafe name if exists
      $str_new_name = capiStrCleanURLCharacters($str_name) . "_" . $int_id;
   } 
		
   // check again - and set name
	if (!in_articles($str_new_name, $int_id, $int_lang_id, $int_idcat)) {
			
      // insert websafe name in article list
      $sql = "UPDATE " . $_SESSION["dbprefix"] . "_art_lang SET urlname = '" . $str_new_name . "' WHERE idart = '" . $int_id . "' AND idlang = '" . $int_lang_id . "'";
      return $db->query($sql);
	} else {
		return false;
   }
} // end function




/**
 * set_category()
 *
 * set websafe name in category list
 *
 * insert new websafe name in category list 
 *
 * @param	string	original name (will be converted)
 * @param	integer	current article id
 * @param	integer current language id
 * @return 	boolean	true if insert was successfully
 */
 function set_category ( $str_name = "", $int_id = 0, $int_lang_id = 0) {
   $db = new DB_Contenido($_SESSION["dbhost"], "", $_SESSION["dbuser"], $_SESSION["dbpass"]);

   if (checkMySQLDatabaseCreation($db, $_SESSION["dbname"])) {
	  $db = new DB_Contenido($_SESSION["dbhost"], $_SESSION["dbname"], $_SESSION["dbuser"], $_SESSION["dbpass"]);
   }
    
   // create websafe name
   $str_new_name = capiStrCleanURLCharacters($str_name);

   // check if websafe name already exists
   if (in_category($str_new_name, $int_id, $int_lang_id)) {
      // create new websafe name if exists
      $str_new_name = capiStrCleanURLCharacters($str_name) . "_" . $int_id;
   }


	// check again - and set name
	if (!in_category($str_new_name, $int_id, $int_lang_id)) {
      // insert websafe name in article list
      $sql = "UPDATE " . $_SESSION["dbprefix"] . "_cat_lang SET urlname = '$str_new_name' WHERE idcat = '$int_id' AND idlang = '$int_lang_id'";
      return $db->query($sql);
   } else {
      return false;
   }
} // end function



	
/**
 * in_articles()
 *
 * Check articles on websafe name
 *
 * Check all articles in the current category 
 * on existing same websafe name
 *
 * @param	string	Websafe name to check
 * @param	integer	current article id
 * @param	integer current language id
 * @param	integer current category id
 * @return 	boolean	true if websafename already exists, false if not
 */
function in_articles ( $str_name = "", $int_id = 0, $int_lang_id = 0, $int_idcat = 0) {
   $db = new DB_Contenido($_SESSION["dbhost"], "", $_SESSION["dbuser"], $_SESSION["dbpass"]);

   if (checkMySQLDatabaseCreation($db, $_SESSION["dbname"])) {
	  $db = new DB_Contenido($_SESSION["dbhost"], $_SESSION["dbname"], $_SESSION["dbuser"], $_SESSION["dbpass"]);
   }

	// handle multipages
   if ($int_idcat > 0) {
      $int_category_id = $int_idcat;
   } else {
      // get category id if not set
      $sql = "SELECT idcat FROM " . $_SESSION["dbprefix"] .	"_cat_art WHERE idart = '$int_id'";
      $db->query($sql);
      $db->next_record();
      $int_category_id = ($db->f("idcat") > 0 ) ? $db->f("idcat") : "0";
   }

   $str_where = " ca.idcat = '$int_category_id' AND";
	$str_where.= " al.idlang = '" . $int_lang_id . "' AND 
                  al.urlname = '" . $str_name . "' AND
                  al.idart <> '$int_id'";

   // check if websafe name is in this category
   $sql = "SELECT count(al.idart) as numcats FROM " . $_SESSION["dbprefix"] .	"_art_lang al LEFT JOIN " . $_SESSION["dbprefix"] .	"_cat_art ca ON al.idart = ca.idart WHERE " . $str_where;
   $db->query($sql);
   $db->next_record();
		
   $int_count = $db->f("numcats");
   	
	return ($int_count > 0) ? true : false;
} // end function



/**
 * in_categories()
 *
 * Check categories on websafe name
 *
 * Check all categories in the main parent 
 * category on existing same websafe name
 *
 * @param	string	Websafe name to check
 * @param	integer	current category id
 * @param	integer current language id
 * @return 	boolean	true if websafename already exists, false if not
 */
function in_category ( $str_name = "", $int_id = 0, $int_lang_id = 0) {
   $db = new DB_Contenido($_SESSION["dbhost"], "", $_SESSION["dbuser"], $_SESSION["dbpass"]);

   if (checkMySQLDatabaseCreation($db, $_SESSION["dbname"])) {
	  $db = new DB_Contenido($_SESSION["dbhost"], $_SESSION["dbname"], $_SESSION["dbuser"], $_SESSION["dbpass"]);
   }
        
   // get parentid
   $sql = "SELECT parentid FROM " . $_SESSION["dbprefix"] .	"_cat WHERE idcat = '$int_id'";
   $db->query($sql);
   $db->next_record();
   $int_parent_id = ($db->f("parentid") > 0 ) ? $db->f("parentid") : "0";

   $str_where = " c.parentid = '$int_parent_id' AND";
   $str_where.= " cl.idlang = '" . $int_lang_id . "' AND 
                  cl.urlname = '" . $str_name . "' AND
                  cl.idcat <> '$int_id'";

   // check if websafe name is in this category
   $sql = "SELECT count(cl.idcat) as numcats FROM " . $_SESSION["dbprefix"] .	"_cat_lang cl LEFT JOIN " . $_SESSION["dbprefix"] . "_cat c ON cl.idcat = c.idcat WHERE " . $str_where;
   $db->query($sql);
   $db->next_record();
		
   $int_count = $db->f("numcats");

   return ($int_count > 0) ? true : false;
} // end function


?>